/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/* ===== accountCentral.css ==========================================
  == Styles for the Messenger Account Central panel.
  ======================================================================= */

@import url("chrome://messenger/skin/");

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

#acctCentralGrid {
  background-color: -moz-Field;
  color: -moz-FieldText;
}

spacer {
  display: none;
}

/* ::::: rows ::::: */

#acctCentralHeaderRow {
  padding-top: 10px;
  padding-bottom: 5px;
  -moz-padding-start: 10px;
  -moz-padding-end: 0px;
  font: menu;
  font-weight: bold;
  font-size: 150%;
  margin: 0px;
}

.acctCentralRow {
  height: 48px;
  margin: 0px 20px !important;
}

.acctCentralRow > hbox {
  margin: 0px 6px;
  padding: 8px 7px;
}


.row-iconic-icon {
  list-style-image: inherit;
  -moz-margin-end: 10px;
}

.acctCentralRowTitleBox {
  background-color: -moz-Dialog;
  border: 1px solid ThreeDDarkShadow;
  color: -moz-DialogText;
  margin: 15px 6px 0px;
  border-top-left-radius: 7px;
  border-top-right-radius: 7px;
  padding-top: 4px;
  padding-bottom: 2px;
  -moz-padding-start: 8px;
  -moz-padding-end: 4px;
  font-weight: bold;
  font: menu;
}

/* ::::: links ::::: */

.acctCentralLinkText {
  cursor: pointer;
  -moz-padding-start: 50px;
  font-size: 11px;
  padding-top: 7px;
  min-height: 32px;
}

.acctCentralLinkText:hover {
  color: -moz-nativehyperlinktext;
  text-decoration: underline;
}

.acctCentralLinkText:active {
  color: -moz-activehyperlinktext;
}

#ReadMessages label {
  background: url("chrome://messenger/skin/accountcentral/mailbox.png")  no-repeat !important;
}

#ComposeMessage label {
  background: url("chrome://messenger/skin/accountcentral/writemail.png")  no-repeat !important;
}

#JunkSettingsMail label {
  background: url("chrome://messenger/skin/accountcentral/junk.png")  no-repeat !important;
}

#SubscribeNewsgroups label {
  background: url("chrome://messenger/skin/accountcentral/manage-subscriptions.png")  no-repeat !important;
}

#AccountSettings label {
  background: url("chrome://messenger/skin/accountcentral/accountsettings.png")  no-repeat !important;
}

#CreateAccount label {
  background: url("chrome://messenger/skin/accountcentral/newaccount.png")  no-repeat !important;
}

#SearchMessages label {
  background: url("chrome://messenger/skin/accountcentral/search-messages.png")  no-repeat !important;
}

#CreateFilters label {
  background: url("chrome://messenger/skin/accountcentral/mailbox.png")  no-repeat !important;
}

#SubscribeImapFolders label {
  background: url("chrome://messenger/skin/accountcentral/folder.png")  no-repeat !important;
}

#SubscribeRSS label {
  background: url("chrome://messenger/skin/accountcentral/manage-rss.png")  no-repeat !important;
}

#OfflineSettings label {
  background: url("chrome://messenger/skin/accountcentral/offline.png")  no-repeat !important;
}
